class CameraParams {
    constructor(api) {
        this.api = api;
        this.paramKeys = {
            serverUrl: 'Camoverlay.ServerUrl',
            appEnabled: 'Camoverlay.Enabled',
            appVersion: 'Camoverlay.Version',
            serialNumber: 'Properties.System.SerialNumber',
            cameraModel: 'Brand.ProdNbr',
            cameraFirmware: 'Properties.Firmware.Version',
            cameraArchitecture: 'Properties.System.Architecture',
            cameraResolutions: 'Properties.Image.Resolution',
            secondaryDNS: 'Network.DNSServer2',
            appVersionHash: 'Camoverlay.VersionHash',
            supportedVersion: 'SupportedVersion',
            soc: 'Soc',
        }
        this.dynamicParamKeys = {
            cameraResolutions: (selectedCamera) => `Properties.Image.I${selectedCamera}.Resolution`,
            cameraResolution: (selectedCamera) => `Image.I${selectedCamera}.Appearance.Resolution`,
            cameraName: (selectedCamera) => `Image.I${selectedCamera}.Name`,
            cameraSource: (videoChannel) => `Image.I${videoChannel}.Source`,
            cameraEnabled: (selectedCamera) => `Image.I${selectedCamera}.Enabled`
        }
    }

    async init() {
        const nbrOfConfigs = parseInt(await this.api.getParameter('Image.NbrOfConfigs'));

        // General params
        let paramsList = [
            this.paramKeys['serverUrl'],
            this.paramKeys['appEnabled'],
            this.paramKeys['appVersion'],
            this.paramKeys['appVersionHash'],
            this.paramKeys['serialNumber'],
            this.paramKeys['cameraArchitecture'],
            this.paramKeys['cameraFirmware'],
            this.paramKeys['cameraResolutions'],
            this.dynamicParamKeys['cameraResolution'](0),
            this.paramKeys['secondaryDNS'],
            this.paramKeys['cameraModel'],
        ];

        // Add image params
        for (let i = 0; i < nbrOfConfigs; i++) {
            paramsList.push(this.dynamicParamKeys['cameraResolutions'](i));
            paramsList.push(this.dynamicParamKeys['cameraResolution'](i));
            paramsList.push(this.dynamicParamKeys['cameraName'](i));
            paramsList.push(this.dynamicParamKeys['cameraSource'](i));
            paramsList.push(this.dynamicParamKeys['cameraEnabled'](i));
        }

        this.params = await this.api.getParameters(paramsList);
    }

    get(paramKey) {
        return this.params[paramKey];
    }

    update(parameters) {
        if (typeof parameters !== 'object' || Array.isArray(parameters) || parameters === null) {
            throw new Error("Argument must be an object");
        }

        for (const [paramKey, paramValue] of Object.entries(parameters)) {
            this.params[paramKey] = paramValue;
        }
    }

    handleUnknownParams(paramKey, params, paramsToRequest) {
        if (this.params[paramKey]) {
            params[paramKey] = this.params[paramKey];
          } else {
            paramsToRequest.push(paramKey);
          }
    }

    getServerUrl() {
        const paramKey = this.paramKeys['serverUrl'];
        return this.get(paramKey);
    }

    getAppEnabled() {
        const paramKey = this.paramKeys['appEnabled'];
        return parseInt(this.get(paramKey));
    }

    getAppVersion() {
        const paramKey = this.paramKeys['appVersion'];
        return this.get(paramKey);
    }

    getSerialNumber() {
        const paramKey = this.paramKeys['serialNumber'];
        return this.get(paramKey);
    }

    getCameraModel() {
        const paramKey = this.paramKeys['cameraModel'];
        return this.get(paramKey);
    }

    getCameraFirmware() {
        const paramKey = this.paramKeys['cameraFirmware'];
        return this.get(paramKey);
    }

    getCameraArchitecture() {
        const paramKey = this.paramKeys['cameraArchitecture'];
        return this.get(paramKey);
    }

    getCameraResolutionsForCamera(selectedCamera = 0) {
        let paramKey = this.dynamicParamKeys['cameraResolutions'](selectedCamera);
        return this.get(paramKey);
    }

    getCameraResolutions() {
        let paramKey = this.paramKeys['cameraResolutions'];
        return this.get(paramKey);
    }

    getSoc() {
        const paramKey = this.paramKeys['soc'];
        return this.get(paramKey);
    }

    setSoc(value) {
        const paramKey = this.paramKeys['soc'];
        this.update({[paramKey]: value});
    }

    getSupportedVersion() {
        const paramKey = this.paramKeys['supportedVersion'];
        return this.get(paramKey);
    }

    setSupportedVersion(value) {
        const paramKey = this.paramKeys['supportedVersion'];
        this.update({[paramKey]: value});
    }

    getCameraResolution(selectedCamera = 0) {
        let paramKey = this.dynamicParamKeys['cameraResolution'](selectedCamera);
        return this.get(paramKey);
    }

    getCameraName(selectedCamera = 0) {
        let paramKey = this.dynamicParamKeys['cameraName'](selectedCamera);
        return this.get(paramKey);
    }

    getCameraSource(videoChannel) {
        let paramKey = this.dynamicParamKeys['cameraSource'](videoChannel);
        return this.get(paramKey);
    }

    getCameraEnabled(selectedCamera = 0) {
        let paramKey = this.dynamicParamKeys['cameraEnabled'](selectedCamera);
        return this.get(paramKey);
    }

    getSecondaryDNS() {
        const paramKey = this.paramKeys['secondaryDNS'];
        return this.get(paramKey);
    }

    getAppVersionHash() {
        const paramKey = this.paramKeys['appVersionHash'];
        return this.get(paramKey);
    }
}