function ControlRoom() {

  /**
   * Init whole Control Room
   * @returns {Promise<void>}
   */
  let init = async function () {

    await renderAll();

    $('#controlRoomClose').click(hideControlRoomCallback);

  }

  const controlRoomHash = '#controlRoom';
  const controlRoomHashX = '#controlRoomX';

  let shouldOpen = () => {
    let shouldOpen = false, hideClose = false;
    const url = new URL(window.location, window.origin);
    if (url.hash === controlRoomHash) {
      shouldOpen = true;
    } else if (url.hash.toLowerCase() === controlRoomHashX.toLowerCase()) {// make x or X both accepted
      shouldOpen = true;
      hideClose = true;
    }
    return [shouldOpen, hideClose];
  }

  let addHash = (hideClose) => {
    const url = new URL(window.location, window.origin);
    const hash = hideClose ? controlRoomHashX : controlRoomHash;
    if (url.hash === hash) {
      return;
    }
    url.hash = hash;
    window.history.pushState('', '', url);
  }

  let removeHash = () => {
    const url = new URL(window.location, window.origin);

    // NOTE: controlRoomHashX is always removed manually,
    // hence this function is only called on 'X' click
    if (url.hash === controlRoomHash) {
      url.hash = '';
      window.history.pushState('', '', url);
    }
  }

  async function updatePTZList(videoChannelId) {
    let cameraSupportsPtz = await supportsPtz();
    if (cameraSupportsPtz) {

      let ptzPositionListJqueryObject = $('#ptzPositionList');
      ptzPositionListJqueryObject.showSpinner();

      try {
        const ptzPositionList = await ptzManager.getList($(videoChannelId).val());

        if (ptzPositionList.length === 0) {
          // If camera has empty view areas, dont load PTZ
          ptzPositionListJqueryObject.hideSpinner();
        } else {
          let ptzCurrentPosition = await ptzManager.getCurrentPosition();
          let ptzPositionHTML = await createPtzPositionHTML(ptzPositionList, ptzCurrentPosition);
          ptzPositionListJqueryObject.html('').append(ptzPositionHTML);

          $('.setPosition').off('click').on('click', async function () {
            $('.setPosition.btn-primary').addClass('btn-secondary').removeClass('btn-primary');
            $(this).addClass('btn-primary');
            let positionName = $(this).attr('data-position-name');
            await ptzManager.setPosition(positionName, $(videoChannelId).val());
          });
        }
      } catch (e) {
        console.error(e.message);
        ptzPositionListJqueryObject.hideSpinner();
      }
    }
  }

  /**
   * Render all components for controlRoom
   */
  async function renderAll() {
    const services = createListOfServices();
    const videoChannelId = '#controlRoom-VideoChannel';
    const resolutionId = '#controlRoom-Resolution';
    const imagePreviewId = 'imagePreviewCamera2';

    // Create video channel

    await createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].cameraList, false);

    // Create resolution list, depend on video channel
    let bestResolution = getBestResolution(services);
    createResolutionList(resolutionId);
    $(resolutionId + ' option:contains(' + bestResolution + ')').attr('selected', 'selected');

    // Refresh preview
    refreshImagePreview();

    await updatePTZList(videoChannelId);

    // Register events
    $(videoChannelId).off('change').on('change', async function () {
      let selectedCamera = parseInt($(this).val());

      try {
        cameraResolutions = createNewResolutionList(
          cameraParams.getCameraResolutionsForCamera(selectedCamera)
          || await vapixApi.getParameter(cameraParams.dynamicParamKeys['cameraResolutions'](selectedCamera))
          || cameraParams.getCameraResolutions()
        );

        resolution = createResolutionObject(
          cameraParams.getCameraResolution(selectedCamera)
          || await vapixApi.getParameter(cameraParams.dynamicParamKeys['cameraResolution'](selectedCamera))
        );
      } catch (error) {
        throw new Error(error.message);
      }

      createResolutionList(resolutionId);
      refreshImagePreview();
      await updatePTZList(videoChannelId);
    });
    $(resolutionId).off('change').on('change', function () {
      refreshImagePreview();
    });

    // Check if is more services than it's allowed
    const exceedsMaxServiceCount = await configurationTmp.exceedsMaxServiceCount();
    if (exceedsMaxServiceCount) {
      const initAlertMessage = await configurationTmp.getMaxAllowedMessage();
      showMessage('info', initAlertMessage, false, 'maxAllowedServices');
    }

    let controlRoomList = $('#controlRoomList');
    controlRoomList.html(''); // Empty list

    // Fill list and attach JS events on each service
    for (let i = 0; i < services.length; i++) {
      const allowedControl = ['manual', 'time'].includes(services[i].automationType);
      if (allowedControl) {
        const serviceForRender = await renderService(services[i]);
        controlRoomList.append(serviceForRender); // Append rendered service to list
        attachScriptToService(services[i]); // Attach events to service
      }
    }

    // After change text value, change textarea with new text
    $('.controlRoomFieldSelect').off('change').on('change', function () {
      var services = createListOfServices();
      var tabName = $(this).attr('data-tab-name');
      var serviceID = parseInt($(this).attr('data-id'));
      var newVal = parseInt($(this).val());
      for (var i = 0; i < services.length; i++) {
        var service = services[i];
        if (service.name === 'textAndBackground' || service.name === 'customGraphics') {
          for (var j = 0; j < service.mappingZones.length; j++) {
            if (service.mappingZones[j].name === tabName && service.id === serviceID) {
              let textareaElementId = $('#textAndBackgroundText-' + serviceID + '-' + tabName + '-InputText');
              let textValue = service.mappingZones[j].text[newVal].source;
              setHeightOfTextarea(textareaElementId, textValue);
              textareaElementId.val(textValue);
            }
          }
        }
      }
    });

    // Event for service actions (show, hide, update)
    $('.serviceAction').off('click').on('click', function (e) {
      e.preventDefault();
      onServiceActionClick($(this));
    });

    $('.countdownEndTime').each(function (index) {
      const countdowns = $('.countdownEndTime');
      let btnId = 'service_Update-' + this.getAttribute('data-id');
      $(countdowns[index]).attachValidation({
        pattern: '(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}(:\\d{2}|))',
        invalidMsg: 'Supported format YYYY-MM-DD HH:MM:SS.',
        disableBtnId: btnId
      });
    });

    $('.changeBtn').off('click').on('click', async function () {
      const serviceId = $(this).data('id');
      const isScreenSharing = $(this).data('type') === 'screenSharing';
      const running = sharingInterface.isRunning(serviceId);
      if (!running) {
        showMessage('warning', 'Screen sharing is currently stopped.');
        return;
      }

      const sharingList = sharingInterface.getList();
      const resolution = sharingList[serviceId].resolution;
      const imageMaxSize = sharingList[serviceId].imageMaxSize;
      const fullscreenOn = sharingList[serviceId].fullscreenOn;
      const fps = sharingList[serviceId].fps;
      const handleOnStop = sharingList[serviceId].onStop;
      const handleSharingPreview = sharingList[serviceId].videoElement.onHidePreview;

      await sharingList[serviceId].stop(serviceId);

      // Start
      const url = new URL(window.location.origin);
      const wsConfig = {
        url: 'wss://' + url.hostname + '/local/camoverlay/service/ws',
        name: isScreenSharing ? 'screen-sharing' : 'pip'
      }
      try {
        await sharingInterface.start(
          isScreenSharing ? 'screen' : 'webcam',
          wsConfig,
          serviceId,
          resolution,
          imageMaxSize,
          fullscreenOn,
          fps,
          handleOnStop,
          () => { },
          handleSharingPreview
        );
      } catch (err) {
        console.error(err);
        let errMsg = isScreenSharing ? 'Screen Sharing: ' : 'Web Camera: ';
        showMessage('danger', errMsg + err.message);
      }
    });

    $('.fullscreenOn, .fullscreenOff').off('click').on('click', function () {
      const serviceId = $(this).data('id');
      const running = sharingInterface.isRunning(serviceId);
      if (!running) {
        showMessage('warning', 'Screen sharing is currently stopped.');
        return;
      }
      const isOn = $(this).hasClass('fullscreenOn');
      sharingInterface.setFullscreen(serviceId, isOn);
      const btnClass = isOn ? '.fullscreenOff' : '.fullscreenOn';
      const btnToShow = $(this).closest('.serviceBtns').find(btnClass + '[data-id="' + serviceId + '"]');
      $(this).hide();
      $(btnToShow).show();
    });

    Sortable.create(controlRoomList[0], {
      handle: '.serviceElementHandle',
      dataIdAttr: 'data-id',
      animation: 150,
      ghostClass: 'draggableElementShadow',
      store: {
        set: function (sortable) {
          saveNewOrderOfServices(sortable.toArray());
        },
      }
    });

    function refreshImagePreview() {
      let videoChannelVal = $(videoChannelId).val();
      let resolutionVal = $(resolutionId).val();
      videoChannelVal = videoChannelVal < 0 ? 0 : videoChannelVal;
      showImageFromCamera(imagePreviewId, cameraResolutions[resolutionVal], videoChannelVal);
    }
  }

  function createPtzPositionHTML(list, currPosition) {
    let htmlOutput = ['<span class="p-1 mr-1">PTZ positions:</span>'];

    list.forEach(item => {
      let active = (item === currPosition) ? 'btn-primary' : 'btn-secondary';
      let htmlItem = '<button class="btn ' + active + ' btn-sm mr-1 setPosition" data-position-name="' + item + '">' + item + '</button>'
      htmlOutput.push(htmlItem);
    });

    return htmlOutput.join('');
  }

  function saveNewOrderOfServices(order) {
    let newServiceArrTmp = [];
    order.forEach(function (item, i) {
      serviceArrTmp.forEach(function (service) {
        if (service.id === parseInt(item)) {
          newServiceArrTmp.push(service);
        }
      });
    });
    serviceArrTmp = [];
    serviceArrTmp = newServiceArrTmp;
    saveBtnCallback(true, newServiceArrTmp);
  }

  /**
   * Get most common and biggest resolution
   * @param {Array} services - array of services
   * @returns {string}
   */
  function getBestResolution(services) {
    // Get resolution from service arr
    let resArr = [];
    services.forEach(function (service) {
      if (service.enabled === 1) {
        resArr.push(service.width + " x " + service.height);
      }
    });

    // Count occurrences of resolution
    let occurrencesArr = [];
    resArr.forEach(function (item) {
      occurrencesArr[item] = occurrencesArr[item] ? ++occurrencesArr[item] : 1;
    });

    // Get max value from occurrencesArr
    let maxValue = Object.values(occurrencesArr).sort().pop();

    // Get all max occurrences
    let allMaxValues = Object.keys(occurrencesArr).filter(key => occurrencesArr[key] === maxValue);

    // Get max width from resolution
    let maxWidth = 0;
    for (let i = 0; i < allMaxValues.length; i++) {
      let currWidth = parseInt((allMaxValues[i].split("x"))[0]);
      maxWidth = maxWidth < currWidth ? currWidth : maxWidth;
    }

    // Get max resolution from arr and return
    return allMaxValues.find(item => item.includes(maxWidth));
  }

  /**
   * Events for service action (show & update, hide, update)
   * @param element - $(this)
   */
  const onServiceActionClick = async (element) => {
    const serviceID = parseInt(element.closest('.serviceElement').attr('data-id'));

    if (element.hasClass('service_ShowAndUpdate')) {
      const canEnableAnotherService = await configurationTmp.canEnableAnotherService();
      if (!canEnableAnotherService) {
        const alertMessage = await configurationTmp.getMaxAllowedMessage();
        showMessage('info', alertMessage, false, 'maxAllowedServices');
      }
      handleShowAndUpdate(serviceID);
    } else if (element.hasClass('service_Hide')) {
      handleHideService(serviceID);
    } else if (element.hasClass('service_Update')) {
      handleUpdateService(serviceID);
    }
  }

  /**
   * @param {number} serviceID
   */
  const handleUpdateService = (serviceID) => {
    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        serviceArrTmp[i] = JSON.parse(JSON.stringify(saveService(serviceArrTmp[i])));
      }
    }
    saveBtnCallback(false);
  }

  /**
   * @param {number} serviceID
   */
  const handleShowAndUpdate = (serviceID) => {
    const serviceElement = 'div.serviceElement[data-id="' + serviceID + '"]';
    $(serviceElement).find('.service_ShowAndUpdate').hide();
    $(serviceElement).find('.service_Hide').show();

    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        // 1. Show
        serviceArrTmp[i].enabled = 1;
        // 2. Update
        serviceArrTmp[i] = JSON.parse(JSON.stringify(saveService(serviceArrTmp[i])));
      }
    }
    saveBtnCallback(true);
  }

  /**
   * @param {number} serviceID
   */
  const handleHideService = (serviceID) => {
    const serviceElement = 'div.serviceElement[data-id="' + serviceID + '"]';
    $(serviceElement).find('.service_Hide').hide();
    $(serviceElement).find('.service_ShowAndUpdate').show();
    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        serviceArrTmp[i].enabled = 0;
      }
    }
    saveBtnCallback(true);
  }

  /** Attach JS script to service
   * @param service - JSON object
   */
  function attachScriptToService(service) {
    var serviceID = service.id;

    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {

      var mappingArea = service.background;
      var mappingAreaElement = $('#textAndBackgroundMappingArea' + serviceID);
      var inputGroupImage = $('#textAndBackgroundImage' + serviceID).closest('.input-group');
      mappingAreaElement.val(mappingArea);
      (mappingArea === 'image') ? inputGroupImage.removeClass('d-none') : inputGroupImage.addClass('d-none');
      // After change mapping area select, show or hide <select> with image list
      mappingAreaElement.on('change', function () {
        mappingArea = $('#textAndBackgroundMappingArea' + serviceID).val();
        (mappingArea === 'image') ? inputGroupImage.removeClass('d-none') : inputGroupImage.addClass('d-none');
      });
      // Set all fields
      for (var i = 0; i < service.mappingZones.length; i++) {
        const labelCountdown = $('#countdown-' + serviceID + "-" + service.mappingZones[i].name);
        const countdownEndTime = $('#countdownEndTime-' + serviceID + "-" + service.mappingZones[i].name + '-InputText');
        const selectElementId = $('#textAndBackgroundSelect-' + serviceID + "-" + service.mappingZones[i].name);
        const textareaElementId = $('#textAndBackgroundText-' + serviceID + "-" + service.mappingZones[i].name + '-InputText');
        if (service.mappingZones[i].type === 'countdown') {
          selectElementId.hide();
          textareaElementId.hide();

          const targetDatetime = convertTimestampToDatetime(service.mappingZones[i].settings.targetDate);
          countdownEndTime.val(targetDatetime);
        } else {
          // Fill select element with options
          var selected = 0;
          var optionList = '';
          for (var o = 0; o < service.mappingZones[i].text.length; o++) {
            if (service.mappingZones[i].text[o].active) selected = o;
            optionList += '<option value="' + o + '">text ' + (o + 1) + '</option>';
          }
          selectElementId
            .html(optionList)
            .val(selected);
          // Fill textarea with value and resize
          var selectValue = selectElementId.val();
          var textValue = service.mappingZones[i].text[parseInt(selectValue)].source;
          setHeightOfTextarea(textareaElementId, textValue);
          textareaElementId.text(textValue);
        }
      }
      // Create list of option for <select>
      createImageList('#textAndBackgroundImage' + serviceID, false, service.image);
    } else if (service.name === 'infoticker') {
      // Resize textarea
      let infotickerSourceElementId = $('#infotickerSource' + service.id);
      let textValue = infotickerSourceElementId.val();
      setHeightOfTextarea(infotickerSourceElementId, textValue);
    } else if (service.name === 'images') {

      const customOverlayBtns = $('.serviceElement[data-id="' + service.id + '"] .custom_overlay_set_active');
      customOverlayBtns.click((event) => {
        const clickedServiceId = Number($(event.target).data('service-id'));
        const activeId = Number($(event.target).data('id'));
        let clickedServiceIndex = 0;
        for (let i = 0; i < serviceArrTmp.length; i++) {
          if (serviceArrTmp[i].id === clickedServiceId) {
            clickedServiceIndex = i;
          }
        }

        customOverlayBtns.each(
          (index, item) => {
            const serviceId = Number($(item).data('service-id'));
            const isActive = activeId === index;
            ButtonControl(item).setActive(isActive);
            saveOverlayInControlRoom(item, isActive, clickedServiceIndex);
          }
        );
        saveBtnCallback(true);
      });
    }

    $("[data-toggle='tooltip'], .toggle-tooltip").not('.no-tooltip').tooltip({ 'delay': { show: 500, hide: 0 } });
  }


  /**
   * @param elem
   * @param {boolean} isActive
   */
  function saveOverlayInControlRoom(elem, isActive, serviceIndex) {
    const elemId = $(elem).data('id');
    const tmpOverlayList = serviceArrTmp[serviceIndex].overlayList[elemId];
    const overlayList = serviceArr[serviceIndex].overlayList[elemId];
    tmpOverlayList.active = isActive;
    const existsInOverlayList = tmpOverlayList === overlayList;
    if (!existsInOverlayList) {
      serviceArr[serviceIndex].overlayList.push(tmpOverlayList);
    }
    overlayList.active = isActive;
  }

  const ButtonControl = (item) => {
    const setActive = (isActive) => {
      const itemJqueryObject = $(item);
      isActive ? itemJqueryObject.removeClass('btn-secondary').addClass('btn-primary')
        : itemJqueryObject.removeClass('btn-primary').addClass('btn-secondary');
    }
    return {
      setActive,
    }
  }

  /**
   * @param textareaElement - textarea jquery element
   * @param textValue - value of textarea
   */
  function setHeightOfTextarea(textareaElement, textValue) {
    let numberOfLines = (textValue.split("\n")).length;
    const charsPerLine = 85; // 85 chars per line
    if (numberOfLines === 1) {
      numberOfLines = Math.ceil((textValue.length * 1.1) / charsPerLine);
    }
    if (textValue.length === 0) {
      numberOfLines = 1;
    }
    if (numberOfLines > 5) { // Max height of textarea is 5 lines
      numberOfLines = 5;
    }
    textareaElement.attr('rows', numberOfLines);
  }

  /** Save service
   * @param service - JSON object
   * @return JSON object
   */
  function saveService(service) {
    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {

      service.background = $('#textAndBackgroundMappingArea' + service.id).val();
      service.image = $('#textAndBackgroundImage' + service.id).val();

      var selects = $('[id^=textAndBackgroundSelect-' + service.id + ']');
      var textareas = $('[id^=textAndBackgroundText-' + service.id + '-]');
      for (var i = 0; i < selects.length; i++) {
        if (service.mappingZones[i].type === 'countdown') {
          const countdownEndTimeTextarea = $('#countdownEndTime-' + service.id + '-' + service.mappingZones[i].name + '-InputText');
          service.mappingZones[i].settings.targetDate = convertDatetimeToTimestamp(countdownEndTimeTextarea.val());
        } else {
          for (var o = 0; o < service.mappingZones[i].text.length; o++) {
            if (o !== parseInt($(selects[i]).val())) {
              service.mappingZones[i].text[o].active = false; // Unset previous active
            } else {
              service.mappingZones[i].text[o].active = true; // Set new active
              service.mappingZones[i].text[o].source = $(textareas[i]).val(); // Set new text value
            }
          }
        }
      }

    } else if (service.name === 'infoticker') {
      service.source = $('#infotickerSource' + service.id).val();
    }
    return service;
  }

  /** Create HTML Service
   * @param service - JSON object
   * @return elementService - string, contains HTML of one service
   */
  async function renderService(service) {
    /** In service 'custom|images' display list of image in collapsed state
     * customList is list of images in service 'custom|images'
     * customClass has empty value or 'custom|images', it helps render service with class .custom height than other services
     */
    let isCustom = ['images', 'custom'].includes(service.name);
    let customList = isCustom ? `<div style="height: auto; width: 100%;">${getCustomOverlay(service)}</div>` : '';
    let customClass = isCustom ? service.name : '';

    // Define service and options
    let elementService = `<div id="serviceElement${service.id}" data-name="${service.name}" data-id="${service.id}" class="m-2 p-0 p-sm-2 mt-0 mb-2 serviceElement border-bottom pb-3 ${customClass}">`;
    let showUpdateBtn = ['infoticker', 'textAndBackground', 'customGraphics'].includes(service.name);
    let updateBtn = showUpdateBtn ? `<button type="button" id="service_Update-${service.id}" class="ml-2 btn btn-sm btn-info service_Update serviceAction">Update</button>` : '';
    let showBtn = `<a href="" class="ml-2 btn btn-sm btn-success service_ShowAndUpdate serviceAction" ${(service.enabled) ? 'style="display:none"' : ''}>Show & Update</a>`;
    let hideBtn = `<a href="" class="ml-2 btn btn-sm btn-danger service_Hide serviceAction" ${(!service.enabled) ? 'style="display:none"' : ''}>Hide</a>`;
    let fullscreenBtns = service.name === 'screenSharing' ? renderFullscreenBtns(service.id) : '';
    let changeBtn = ['screenSharing', 'pip'].includes(service.name) ? await renderChangeBtn(service.id, service.name) : '';
    let serviceNameFull = service.customName ? escapeHtml(service.customName) : getFullNameOfService(service.name);
    let inputs = renderInputs(service);

    let itemHeader = '<div class="serviceHeader">';
    itemHeader += '<button class="serviceElementHandle mr-2 btn btn-sm btn-link"><span class="fa fa-grip-vertical"></span></button>'; //Grab handler
    if (inputs !== "") { // Link for collapsing only if inputs are not empty
      itemHeader += `<a class="serviceName btn btn-link mr-2 pl-0" data-toggle="collapse" data-target="#itemBody${service.id}" role="button" aria-expanded="false" aria-controls="itemBody${service.id}">`;
    } else {
      itemHeader += '<a class="serviceName containerText btn btn-link mr-2 pl-0" style="cursor:default;">';
    }
    itemHeader += `<div class="serviceNameId font-weight-bold" data-name="${serviceNameFull}" data-id="${service.id}">${serviceNameFull}</div><div class="ml-1 text-muted">(ID: ${service.id})</div>`; // Service name with ID
    itemHeader += (inputs !== "") ? '<div class="ml-2 fas fa-chevron-down iconPlus"></div><div class="ml-2 fas fa-chevron-up iconMinus"></div>' : ''; // Arrows for collapsing
    itemHeader += '</a>';
    itemHeader += customList + '</div>';

    let innerDivInBody = `<div style="${(inputs !== "") ? 'padding-bottom: 40px;' : ''}">${inputs}</div>`;
    let itemBody = `<div class="collapse" id="itemBody${service.id}">${innerDivInBody}</div>`;
    let itemFooter = `<div class="serviceFooter"><div class="serviceBtns text-right mt-1 mr-3">${updateBtn}${fullscreenBtns}${changeBtn}${showBtn}${hideBtn}</div></div>`;

    elementService += itemHeader + itemBody + itemFooter;
    elementService += '</div>';

    return elementService;
  }

  async function renderChangeBtn(serviceId, serviceName) {
    const isRunnig = sharingInterface.isRunning(serviceId);
    const isSharingScreen = serviceName === 'screenSharing';
    const allowChangeCamera = await sharingInterface.isAllowedChangeCamera(serviceId);
    const isAllowed = isSharingScreen ? true : allowChangeCamera;
    const btnShow = (isRunnig && isAllowed) ? 'inline-block' : 'none';
    const btnText = isSharingScreen ? 'Change Screen' : 'Change Camera'
    const changeBtn = `<button data-id="` + serviceId + `" data-type="` + serviceName + `" type="button" data-toggle="tooltip" title="` + btnText + `" class="changeBtn ml-2 btn btn-sm" style="display:` + btnShow + `">
        <span class="fas fa-sync-alt"></span>
      </button>`;
    return changeBtn;
  }

  function renderFullscreenBtns(serviceId) {
    const running = sharingInterface.isRunning(serviceId);
    const fullscreenIsOn = running ? sharingInterface.getFullscreen(serviceId) : false;
    let btns;
    if (!running) {
      btns = { fullscreenOn: 'none', fullscreenOff: 'none' };
    } else {
      btns = fullscreenIsOn ? { fullscreenOn: 'none', fullscreenOff: 'inline-block' } : { fullscreenOn: 'inline-block', fullscreenOff: 'none' };
    }

    const fullscreenBtnOn = `<button data-id="` + serviceId + `" type="button" data-toggle="tooltip" title="Fullscreen" class="fullscreenOn ml-2 btn btn-sm" style="display:` + btns.fullscreenOn + `">
        <div class="fa-1x">
            <span class="fa-layers fa-fw">
                <i class="fas fa-slash fa-rotate-90"></i>
                <i class="fas fa-expand" data-fa-transform="shrink-2"></i>
            </span>
        </div>
      </button>`;
    const fullscreenBtnOff = `<button data-id="` + serviceId + `" type="button" data-toggle="tooltip" title="Fullscreen" class="fullscreenOff ml-2 btn btn-sm btn-primary" style="display:` + btns.fullscreenOff + `">
        <div class="fa-1x">
            <span class="fa-layers fa-fw">
                <i class="fas fa-slash fa-rotate-90" style="display:none;"></i>
                <i class="fas fa-expand" data-fa-transform="shrink-2"></i>
            </span>
        </div>
      </button>`;

    return fullscreenBtnOn + fullscreenBtnOff;
  }

  /** Render different types of inputs for each service
   * @param service is json object
   * @return string of HTML with inputs
   */
  function renderInputs(service) {
    var inputs = '';
    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {
      inputs = getCustomGraphics(service);
    } else if (service.name === 'infoticker') {
      inputs = (service.sourceType === 'text') ? getInfoTicker(service) : '';
    } //missing service 'custom' because it's rendered in renderService
    return inputs;
  }

  /** Create HTML inputs for Custom Overlay
   * if triggering - automatic - display list of images
   * if triggering - manual - display list of btns for control active overlay
   * @param service is json object
   * @return string of HTML for service 'customOverlay'
   */
  function getCustomOverlay(service) {
    let defaultHtmlAutomatic = '<span class="text-muted" style="padding-left: 5px; padding-right: 200px; white-space: nowrap;text-overflow: ellipsis;width: 100%;display: block;overflow: hidden;">%REPLACE%</span>';
    let defaultHtmlManual = '<div style="margin-bottom: 36px;">%REPLACE%</div>';
    let items = [];
    let overlayList = service.overlayList;
    let isManual = service.automationType === 'manual';
    for (let i = 0; i < overlayList.length; i++) {
      if (isManual) {
        let imgName = handleImageName(overlayList[i].imgName);
        let btnClasses = overlayList[i].active ? 'btn-primary' : 'btn-secondary';
        const btnHtml = '<btn class="custom_overlay_set_active btn btn-sm ' + btnClasses + ' ml-1 mb-1" data-service-id="' + service.id + '" data-id="' + i + '">' + imgName + '</btn>';
        items.push(btnHtml);
      } else {
        items.push(overlayList[i].imgName);
      }
    }
    return isManual ? defaultHtmlManual.replace('%REPLACE%', items.join('')) : defaultHtmlAutomatic.replace('%REPLACE%', items.join(', '));
  }

  const handleImageName = (imgName) => {
    if (imgName.length > 15) {
      const fileType = imgName.split(".")[1];
      imgName = imgName.slice(0, 15) + '...' + fileType;
    }
    return imgName;
  }


  /** Create HTML inputs for Info Ticker
   * @param service is json object
   * @return string of HTML for service 'infoTicker'
   */
  function getInfoTicker(service) {
    var all = $('#patternServices > #infoTicker').html();
    let serviceSource = replaceSpecialChars(service.source);
    all = all.replace(/%ID%/g, service.id);
    all = all.replace(/%VALUE%/g, serviceSource);
    return all;
  }

  /** Create HTML inputs for customGraphics (textAndBackground)
   * @param service is json object
   * @return string of HTML for service 'customGraphics'
   */
  function getCustomGraphics(service) {
    var all = $('#patternServices > #textAndBackground').html();
    var allFields = '';

    allFields += '<div class="col"><form>';
    for (var i = 0; i < service.mappingZones.length; i++) {
      let mappingZoneName = replaceSpecialChars(service.mappingZones[i].name);
      var textLabel = '<div for="textAndBackgroundText%TAB_NAME%InputText" class="input-group-prepend"><span class="input-group-text">' + mappingZoneName + '</span></div>';
      const countdownLabel = '<div class="input-group-prepend" id="countdown-%ID%-%TAB_NAME%"><span class="input-group-text">' + 'End Time' + '</span></div>';
      const countdownField = '<textarea class="form-control countdownEndTime" rows="1" id="countdownEndTime-%ID%-%TAB_NAME%-InputText" data-id="%ID%" data-tab-name="%TAB_NAME%"></textarea>';
      var selectSource = '<select class="form-control controlRoomFieldSelect" id="textAndBackgroundSelect-%ID%-%TAB_NAME%" data-id="%ID%" data-tab-name="%TAB_NAME%"></select>';
      var textField = '<textarea class="form-control controlRoomFieldTextarea" rows="1" id="textAndBackgroundText-%ID%-%TAB_NAME%-InputText" placeholder="Use enter to break the lines"></textarea>';
      var inputGroup = '<div class="input-group mt-1">';
      inputGroup += textLabel.replace(/%TAB_NAME%/g, mappingZoneName);
      if (service.mappingZones[i].type === 'countdown') {
        inputGroup += countdownLabel.replace(/%TAB_NAME%/g, mappingZoneName) + countdownField.replace(/%TAB_NAME%/g, mappingZoneName);
      }
      inputGroup += selectSource.replace(/%TAB_NAME%/g, mappingZoneName)
      inputGroup += textField.replace(/%TAB_NAME%/g, mappingZoneName)
      inputGroup += '</div>';
      allFields += inputGroup;
    }
    allFields += '</form></div>';

    all += allFields;
    all = all.replace(/%ID%/g, service.id);
    return all;
  }

  /** Helper fce for service name
   * @param {string} name - string
   * @return {string} serviceName - nice name of service
   */
  function getFullNameOfService(name) {
    const serviceName = {
      accuweather: 'Weather (AccuWeather)',
      custom: 'Images',
      images: 'Images',
      ptz: 'PTZ Overlay',
      ptzCompass: 'PTZ Compass',
      infoticker: 'Info Ticker',
      textAndBackground: 'Custom Graphics',
      customGraphics: 'Custom Graphics',
      pip: 'Picture In Picture',
      scripter: 'Scripter',
      screenSharing: 'Screen Sharing',
    };
    return serviceName[name];
  }

  /** Create array of services
   * @return {Array} - array of services
   */
  function createListOfServices() {
    var services = [];
    for (var i = 0; i < serviceArrTmp.length; i++) {
      services.push(serviceArrTmp[i]);
    }
    return services;
  }

  return {
    init,
    shouldOpen,
    addHash,
    removeHash,
  }

}