class RequestManager {
    async get(url, cache = 'no-cache', mode = 'cors') {
        const response = await fetch(url, {
            method: 'GET',
            cache,
            mode,
        });

        return response;
    }

    async post(url, payload, cache = 'no-cache', mode = 'cors', headers = new Headers({
        'Content-Type': 'application/x-www-form-urlencoded',
    })) {
        const response = await fetch(url, {
            method: 'POST',
            cache,
            mode,
            headers,
            body: payload,
        });

        return response;
    }

    async postJsonData(url, params, headers = {}, cache = 'no-cache', mode = 'cors') {
        const allHeaders = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            ...headers
        }
        return await this.post(url, JSON.stringify(params), cache, mode, allHeaders);
    }

    async postUrlEncoded(url, params, headers = {}, cache = 'no-cache', mode = 'cors') {
        const allHeaders = {
            'Content-Type': 'application/x-www-form-urlencoded',
            ...headers
        }
        return await this.post(url, this.buildQuery(params), cache, mode, allHeaders);
    }

    async postViaProxyUrlEncoded(proxyUrl, params, headerParams) {
        const headers = this.buildAndCheckProxyHeaders(headerParams);
        return await this.postUrlEncoded(proxyUrl, params, headers);
    }

    async postViaProxyJsonData(proxyUrl, params, headerParams) {
        const headers = this.buildAndCheckProxyHeaders(headerParams);
        return await this.postJsonData(proxyUrl, params, headers);
    }

    buildAndCheckProxyHeaders(headerParams) {
        //check
        const requiredHeaders = [
            'x-target-camera-ip',
            'x-target-camera-port',
            'x-target-camera-user',
            'x-target-camera-pass',
            'x-target-camera-path',
        ];

        const missingHeader = utils.objectHasRequiredKeys(requiredHeaders, headerParams);
        if (missingHeader !== null) {
            throw new Error(`${missingHeader} header is missing.`);
        }

        const unsetValueKey = utils.objectHasValuesSet(headerParams);
        if (unsetValueKey !== null) {
            throw new Error(`${unsetValueKey} header is not set.`);
        }

        return {
            'x-target-camera-ip': headerParams['x-target-camera-ip'],
            'x-target-camera-port': headerParams['x-target-camera-port'],
            'x-target-camera-user': headerParams['x-target-camera-user'],
            'x-target-camera-pass': headerParams['x-target-camera-pass'],
            'x-target-camera-path': headerParams['x-target-camera-path'],
        }
    }

    async postFormData(url, params, cache = 'no-cache', mode = 'cors') {
        const data = params instanceof FormData ? params : this.buildFormData(params);

        // Do not set header to multipart/form-data - not working
        // allow it to be set automatically
        return await this.post(url, data, cache, mode, new Headers({}));
    }

    buildQuery(params) {
        const query = [];
        for (const [key, value] of Object.entries(params)) {
            query.push(key + "=" + encodeURIComponent(value))
        }
        return query.join("&");
    }

    buildFormData(params) {
        const data = new FormData();

        for (const [key, value] of Object.entries(params)) {
            data.append(key, value);
        }

        return data;
    }

    parseResponse(string) {
        const params = {};
        const lines = string.split('\n');
        const equal_string = '=';
        let parts;

        lines.forEach(line => {
            if (line[0] !== equal_string && line[line.length - 1] !== equal_string && line.length > 0) {
                parts = line.split(equal_string);
                params[String(parts[0]).trim()] = String(parts[1]).trim();
            }
        });
        return params;
    }

    async handleParseTextResponseError(response) {
        try {
            return await response.text();
        } catch (error) {
            console.error('Error parsing text response');
        }
    }

    async handleParseJsonResponseError(response) {
        try {
            return await response.json();
        } catch (error) {
            console.error('Error parsing json response');
        }
    }

    async handleParseBlobResponseError(response) {
        try {
            return await response.blob();
        } catch (error) {
            console.error('Error parsing blob response');
        }
    }
}