const WS_RECONNECT_INTERVAL = 3000;

function SharingInterface() {

  const sharingInterfaceContainer = $('#sharingInterfaceContainer');
  let list = {};

  let getWsToken = async () => {
    try {
      let tokenResponse = await camOverlayApi.wsAuthorization();
      if (tokenResponse.status !== 200) {
        console.error("SharingInterface - Request failed (WS authorization token)");
        return null;
      }

      let tokenJson = await tokenResponse.json();
      if (tokenJson.status != 200 || !tokenJson.message) {
        console.error("SharingInterface - Invalid JSON response (WS authorization token)");
        return null;
      }
      return tokenJson.message;

    } catch (err) {
      console.error("SharingInterface - Can't get WS authorization token: " + err.message);
      return null;
    }
  }

  let eventCodeToMsg = (eventCode) => {
    switch (eventCode) {
      case 1000: return 'Normal Closure';
      case 1001: return 'Going Away';
      case 1002: return 'Protocol error';
      case 1003: return 'Unsupported Data';
      case 1004: return 'Reserved';
      case 1005: return 'No Status Rcvd';
      case 1006: return 'Abnormal closure';
      case 1007: return 'Invalid frame payload data';
      case 1008: return 'Policy Violation';
      case 1009: return 'Message Too Big';
      case 1010: return 'Mandatory Ext.';
      case 1011: return 'Internal Server Error';
      case 1015: return 'TLS handshake';
      default: return "Code:" + eventCode;
    }
  }

  let wsConnection = (token, url, name, serviceId) => {
    return new Promise((resolve, reject) => {
      let ws = new WebSocket(url, name);
      ws.binaryType = 'blob';
      ws.onopen = () => {
        ws.send(JSON.stringify({
          authorization: token,
          serviceID: serviceId
        }));
        $('#container_videoSource' + serviceId).removeClass('reconnecting');
        resolve(ws);
      }
      ws.onclose = (event) => {
        if (existInList(serviceId)) {
          $('#container_videoSource' + serviceId).addClass('reconnecting');
          list[serviceId].timeoutInterval = setTimeout(() => {
            wsReconnect(url, name, serviceId);
          }, WS_RECONNECT_INTERVAL);
        } else {
          const msg = eventCodeToMsg(event.code);
          reject(Error(msg));
        }
      }
    });
  }

  let wsReconnect = async (url, name, serviceId) => {
    if (!existInList(serviceId)) {
      return;
    }

    const token = await getWsToken();
    if (!token) {
      list[serviceId].timeoutInterval = setTimeout(() => {
        wsReconnect(url, name, serviceId);
      }, WS_RECONNECT_INTERVAL);
      return;
    }
    try {
      list[serviceId].ws = await wsConnection(token, url, name, serviceId);
    } catch (err) {
      console.error('SharingInterface - ws reconnect: ' + err.message, serviceId)
    }
  }

  let wsIsOpened = (serviceId) => {
    if (!existInList(serviceId)) {
      return false;
    }
    return list[serviceId].ws.readyState === WebSocket.OPEN;
  }

  /**
   * Create CANVAS with captured picture
   * @returns {HTMLCanvasElement}
   */
  let canvasWithPicture = (videoElement, capture, resolution, imageMaxSize, fullscreenOn) => {
    return new Promise((resolve, reject) => {
      const video = videoElement.get();
      const captureObject = capture.getSettings();
      const currentResolution = resolution;
      const currentImageMaxSize = imageMaxSize;
      const isFullscreenOn = fullscreenOn;

      const canvas = document.createElement("canvas");
      canvas.height = isFullscreenOn ? currentResolution.height : currentImageMaxSize;
      canvas.width = isFullscreenOn ? currentResolution.width : currentImageMaxSize;

      const ratio = Math.min(canvas.width / captureObject.width, canvas.height / captureObject.height);
      const newWidth = captureObject.width * ratio;
      const newHeight = captureObject.height * ratio;

      if (!isFullscreenOn) {
        if (newHeight >= newWidth) {
          canvas.width = newWidth;
        } else {
          canvas.height = newHeight;
        }
      }

      let offsetX = isFullscreenOn ? (canvas.width - newWidth) / 2 : 0;
      let offsetY = isFullscreenOn ? (canvas.height - newHeight) / 2 : 0;
      canvas.getContext('2d').drawImage(video, offsetX, offsetY, newWidth, newHeight);
      canvas.toBlob(resolve, 'image/jpeg');
    });
  }

  let wsSendInInterval = (serviceId) => {
    if (!existInList(serviceId)) {
      return;
    }

    // Stop already running worker
    if (list[serviceId].sendingIntervalWorker) {
      list[serviceId].sendingIntervalWorker.terminate();
    }

    // Start new worker
    list[serviceId].sendingIntervalWorker = new Worker('js/WorkerInterval.js');
    list[serviceId].sendingIntervalWorker.onmessage = async (evt) => {
      const imageData = await canvasWithPicture(
        list[serviceId].videoElement,
        list[serviceId].capture,
        list[serviceId].resolution,
        list[serviceId].imageMaxSize,
        list[serviceId].fullscreenOn
      );
      wsSend(serviceId, imageData);
    };
    list[serviceId].sendingIntervalWorker.postMessage(list[serviceId].fps);
  }

  let wsSend = (serviceId, data) => {
    if (wsIsOpened(serviceId)) {
      list[serviceId].ws.send(data);
    }
  }

  let wsFullscreenOn = (serviceId) => wsSend(serviceId, JSON.stringify({command: "ENABLE_FULLSCREEN"}));
  let wsFullscreenOff = (serviceId) => wsSend(serviceId, JSON.stringify({command: "DISABLE_FULLSCREEN"}));
  let wsClose = (serviceId) => list[serviceId].ws.close();

  let containerShowOrHide = () => {
    Object.keys(list).length === 0 ? sharingInterfaceContainer.removeClass('open') : sharingInterfaceContainer.addClass('open');
  }

  let setResolution = (serviceId, currentResolution) => {
    if (!existInList(serviceId)) {
      return;
    }
    list[serviceId].resolution = currentResolution;
  }

  let setFullscreen = (serviceId, fullscreen) => {
    if (!existInList(serviceId)) {
      return;
    }
    list[serviceId].fullscreenOn = fullscreen;
    list[serviceId].fullscreenOn ? wsFullscreenOn(serviceId) : wsFullscreenOff(serviceId);
  }
  let setImageMaxSize = (serviceId, imageMaxSize) => {
    if (!existInList(serviceId)) {
      return;
    }
    list[serviceId].imageMaxSize = imageMaxSize;
  }

  let setFps = (serviceId, framePerSeconds) => {
    if (!existInList(serviceId)) {
      return;
    }
    list[serviceId].fps = framePerSeconds;
    list[serviceId].capture.setFps(framePerSeconds);
    wsSendInInterval(serviceId);
  }
  let getFullscreen = (serviceId) => list[serviceId].fullscreenOn;
  let getImageMaxSize = (serviceId) => list[serviceId].imageMaxSize;
  let getFps = (serviceId) => list[serviceId].fps;
  let isRunning = (serviceId) => existInList(serviceId);
  let isPreviewVisible = async (serviceId) => {
    if (list[serviceId] == undefined || !list[serviceId].hasOwnProperty('videoElement')) {
      return false;
    }
    const isVisible = await list[serviceId].videoElement.isPreviewVisible();
    return isVisible;
  }
  let isAllowedChangeCamera = async (serviceId) => {
    if (list[serviceId] == undefined || !list[serviceId].hasOwnProperty('capture')) {
      return false;
    }
    const webcamList = await list[serviceId].capture.getWebcamList();
    const allowed = webcamList.length > 1;
    return allowed;
  }

  let start = async (type, wsConfig, serviceId, resolution, imageMaxSize, fullscreenOn, fps, onStopCallback, onChangeOptionCallback, onHidePreviewCallback) => {
    if (!window.Worker) {
      throw Error("Unsupported browser (Worker support is needed).");
    }

    const token = await getWsToken();
    if (!token) {
      throw Error("Can't get WS authorization token");
    }
    const websocket = await wsConnection(token, wsConfig.url, wsConfig.name, serviceId);

    let capture = new Capture();
    let mediaStream;
    try {
      if (type === 'screen') {
        mediaStream = await capture.setScreen(fps);
      } else if (type === 'webcam') {
        let deviceList = await capture.getWebcamList();
        let deviceId = await handleChooseDeviceId(deviceList).catch((err) => {
          throw Error('Source was not selected');
          return;
        });
        if (deviceId === -1) {
          throw Error('Source was not selected');
          return;
        }
        if (deviceList.length > 1) {
          onChangeOptionCallback();
        }
        mediaStream = await capture.setWebcam(fps, deviceId);
      } else {
        throw Error('Unknown source');
      }
    } catch (err) {
      throw Error(err.message);
    }

    const videoElement = new VideoElement(serviceId, sharingInterfaceContainer);
    await videoElement.create(mediaStream);

    videoElement.onLoadedData(() => { wsSendInInterval(serviceId); });
    videoElement.onStop(serviceId, () => { stop(serviceId); });
    videoElement.onHidePreview(onHidePreviewCallback);
    capture.onEnded(() => { stop(serviceId); });

    const item = {
      service_id: serviceId,
      resolution: resolution,
      imageMaxSize: imageMaxSize,
      fullscreenOn: fullscreenOn,
      fps: fps,
      ws: websocket,
      capture: capture,
      videoElement: videoElement,
      sendingIntervalWorker: null,
      timeoutInterval: null,
      onStop: onStopCallback,
      stop: stop
    };
    list[serviceId] = item;

    containerShowOrHide();
  }
  
  let handleChooseDeviceId = async (deviceList) => {
    return new Promise((resolve, reject) => {
      if (deviceList.length === 1) {
        resolve(deviceList[0].deviceId);
        return;
      }
      if (deviceList.length === 0) {
        resolve(-1);
        return;
      }
      let outputHTML = '<table class="deviceListTable rounded table table-sm table-borderless table-hover">';
      deviceList.forEach(device => {
        outputHTML += '<tr>';
        outputHTML += '<td>' + device.label + '</td>';
        outputHTML += '<td>';
        outputHTML += '<button class="chooseDeviceIdBtn btn btn-primary" data-device-id="' + device.deviceId + '">Choose</button>';
        outputHTML += '</td>';
        outputHTML += '<tr>';
      });
      outputHTML += '</table>';
      const deviceListModal = $('#chooseDeviceId');
      deviceListModal.find('.modal-body').html(outputHTML);
      deviceListModal.modal();

      deviceListModal.find('.close').off('click').on('click', function() {
        resolve(-1);
        return;
      });
      deviceListModal.find('.chooseDeviceIdBtn').off('click').on('click', function() {
        let deviceId = $(this).data('device-id');
        deviceListModal.modal('hide');
        resolve(deviceId);
      });
    });
  }

  let showPreview = async (serviceId) => {
    if (!existInList(serviceId)) {
      return;
    }
    list[serviceId].videoElement.showPreview();
  }

  let stop = (serviceId) => {
    if (!existInList(serviceId)) {
      return;
    }

    if (list[serviceId].sendingIntervalWorker) {
      list[serviceId].sendingIntervalWorker.terminate();
    }
    list[serviceId].capture.stop();
    list[serviceId].videoElement.remove();
    list[serviceId].onStop();
    wsClose(serviceId);
    delete list[serviceId];

    containerShowOrHide();
  }

  let getList = () => list;

  let existInList = (serviceId) => {
    return list[serviceId] !== undefined;
  }

  return {
    isRunning,
    isPreviewVisible,
    isAllowedChangeCamera,
    start,
    stop,
    showPreview,
    setResolution,
    setFullscreen,
    setImageMaxSize,
    setFps,
    getFullscreen,
    getImageMaxSize,
    getFps,
    getList,
  }
}