class CamOverlayApi {
    constructor(requestManager) {
        this.requestManager = requestManager;
    }

    async getCameraTime() {
        let response;
        try {
            response = await this.requestManager.get('/local/camoverlay/api/camera_time.cgi');
        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getServices() {
        let response;
        try {
            response = await this.requestManager.get('/local/camoverlay/api/services.cgi?action=get', 'default');
        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Getting services failed');
        }

        response = await this.requestManager.handleParseJsonResponseError(response);
        return response.services;
    }

    async setServices(services) {
        let response;
        try {
            response = await this.requestManager.post(
                '/local/camoverlay/api/services.cgi?action=set',
                services,
                'default'
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        response = await this.requestManager.handleParseJsonResponseError(response);
        if (response.status !== 200) {
            throw new Error('Setting services failed');
        }
        return response;
    }

    async report(payload) {
        let response;
        try {
            response = await this.requestManager.postFormData(
                '/local/camoverlay/api/report.cgi',
                payload
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Failed to fetch report');
        }
        return response;
    }

    async wsAuthorization() {
        try {
            return await this.requestManager.get('/local/camoverlay/api/ws_authorization.cgi');

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }
    }

    async getRemoteCameraParams(params, headerParams) {
        let response;
        try {
            response = await this.requestManager.postViaProxyUrlEncoded(
                '/local/camoverlay/api/proxy.cgi',
                params,
                {
                    ...headerParams,
                    'x-target-camera-path': '/axis-cgi/param.cgi',
                }
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Connection error - cannot connect to proxy camera');
        }

        response = await this.requestManager.handleParseTextResponseError(response);
        return this.requestManager.parseResponse(response);
    }

    async getRemoteDeviceInfo(params, headerParams) {
        let response;
        try {
            response = await this.requestManager.postViaProxyJsonData(
                '/local/camoverlay/api/proxy.cgi',
                params,
                {
                    ...headerParams,
                    'x-target-camera-path': '/axis-cgi/basicdeviceinfo.cgi',
                }
            );

        } catch (error) {
            throw new Error(error.message);
        }

        if (!response.ok) {
            throw new Error('Connection error - cannot connect to remote camera');
        }

        response = await this.requestManager.handleParseJsonResponseError(response);
        if (!response.data) {
            throw new Error(response.error.message);
        }

        return response.data;
    }

    async removeFile(params) {
        try {
            await this.requestManager.postUrlEncoded('/local/camoverlay/api/upload_font.cgi?action=remove', params);

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }
    }

    async removeImage(params) {
        try {
            await this.requestManager.postUrlEncoded('/local/camoverlay/api/upload_image.cgi?action=remove', params);

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }
    }

    async uploadFile(payload) {
        try {
            await this.requestManager.postFormData(
                '/local/camoverlay/api/upload_font.cgi?action=upload',
                payload
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }
    }

    async uploadImage(payload) {
        try {
            await this.requestManager.postFormData(
                '/local/camoverlay/api/upload_image.cgi?action=upload',
                payload
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }
    }

    async getFontList() {
        let response;
        try {
            response = await this.requestManager.get(
                '/local/camoverlay/api/upload_font.cgi?action=list'
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Getting font list failed');
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getImageList() {
        let response;
        try {
            response = await this.requestManager.get(
                '/local/camoverlay/api/upload_image.cgi?action=list'
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Getting image list failed');
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getFontStorage() {
        let response;
        try {
            response = await this.requestManager.get(
                '/local/camoverlay/api/upload_font.cgi?action=get_storage'
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Getting font storage failed');
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }

    async getImageStorage() {
        let response;
        try {
            response = await this.requestManager.get(
                '/local/camoverlay/api/upload_image.cgi?action=get_storage'
            );

        } catch (error) {
            throw new Error(`${error.name}: ${error.message}`);
        }

        if (!response.ok) {
            throw new Error('Getting image storage failed');
        }

        return await this.requestManager.handleParseJsonResponseError(response);
    }
}