class VapixApi {
    constructor(requestManager) {
        this.requestManager = requestManager;
    }

    async getParameter(name) {
        let parameters = await this.getParameters([name]);
        return parameters[name];
    };

    async getParameters(parameters) {
        if (typeof parameters !== 'object' || parameters === null) {
            throw new Error("Argument must be an object or Array");
        }

        if (Array.isArray(parameters)) {
            parameters = Object.fromEntries(parameters.map(name => [name, name]));
        }

        let param_names = Object.values(parameters);
        if (param_names.some(name => typeof name === "string" && name.match(/\.StreamList/i)) && param_names.length !== 1) {
            throw new Error("Can't read parameters containing newlines together with another parameter.");
        }

        let group = param_names.join(',');
        let params = {
            dummy: new Date().getTime(),
            action: 'list',
            group: group
        };

        let response;
        try {
            response = await this.requestManager.postUrlEncoded("/axis-cgi/param.cgi", params);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        response = await this.requestManager.handleParseTextResponseError(response);

        if (typeof response !== 'string' || response === '') {
            throw new Error('Empty response from camera');
        }

        // Parse and create object
        return this.requestManager.parseResponse(response);
    };

    async setParameters(parameters) {
        if (typeof parameters !== 'object' || Array.isArray(parameters) || parameters === null) {
            throw new Error("Argument must be an object");
        }

        const query_params = parameters;
        query_params['action'] = 'update';

        let response;
        try {
            response = await this.requestManager.postUrlEncoded("/axis-cgi/param.cgi", query_params);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        response = await this.requestManager.handleParseTextResponseError(response);

        if (response !== 'OK') {
            throw new Error('Error setting parameter');
        }
    }

    async getBasicDeviceInfo(params) {
        if (typeof params !== 'object' || Array.isArray(params) || params === null) {
            throw new Error("Argument must be an object");
        }
        let response;
        try {
            response = await this.requestManager.postJsonData('/axis-cgi/basicdeviceinfo.cgi', params);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        if (!response.ok) {
            throw new Error('Error getting supported version data');
        }

        response = await this.requestManager.handleParseJsonResponseError(response);
        if (!response.data) {
            throw new Error(response.error.message);
        }

        return response.data;
    }

    async getPtzData(params) {
        if (typeof params !== 'object' || Array.isArray(params) || params === null) {
            throw new Error("Argument must be an object");
        }

        let response;
        try {
            response = await this.requestManager.postUrlEncoded("/axis-cgi/com/ptz.cgi", params);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        if (!response.ok) {
            throw new Error('Error getting ptz data');
        }

        return await this.requestManager.handleParseTextResponseError(response);
    }

    async getParsedPtzData(params) {
        const response = await this.getPtzData(params);
        return this.requestManager.parseResponse(response);
    }
    
    async goToPtzPosition(params) {
        if (typeof params !== 'object' || Array.isArray(params) || params === null) {
            throw new Error("Argument must be an object");
        }

        if (typeof params['gotoserverpresetname'] === 'undefined') {
            throw new Error("Parameter 'gotoserverpresetname' not set");
        }

        await this.requestManager.postUrlEncoded("/axis-cgi/com/ptz.cgi", params);
    }

    async startAcap() {
        let response;
        try {
            response = await this.requestManager.get('/axis-cgi/applications/control.cgi?package=camoverlay&action=start')
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        return await this.requestManager.handleParseTextResponseError(response);
    }

    async getIOPortParameter(parameters) {
        if (typeof parameters !== 'object' || parameters === null) {
            throw new Error("Argument must be an object or Array");
        }

        if (Array.isArray(parameters)) {
            parameters = Object.fromEntries(parameters.map(name => [name, name]));
        }

        let response;
        try {
            response = await this.requestManager.postUrlEncoded('/axis-cgi/io/port.cgi', parameters);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        if (!response.ok) {
            throw new Error(`Failed to fetch IOPort parameter`);
        }

        response = await this.requestManager.handleParseTextResponseError(response);
        return this.requestManager.parseResponse(response);
    }

    async getServerReport(mode = 'text') {
        let response;
        try {
            response = await this.requestManager.get(`/axis-cgi/admin/serverreport.cgi?mode=${mode}`);
        } catch (error) {
            throw new Error(`HTTP ${error.status}: ${error.statusText}`);
        }

        if (!response.ok) {
            throw new Error(`Failed to fetch server report`);
        }

        return await this.requestManager.handleParseBlobResponseError(response);
    }
}